@@ -35,9 +35,8 @@ typedef struct {
     u32 crc;
     byte helpbuf[100];
     int  helpidx, helplen;
-    byte tempbuf[100];
-    int  tempidx, templen;
-    void *fake;
+    int faked;
+    int parse_state;
     int inp_checked;   /* set if inp has been checked */
     int inp_bypass;    /* set if the input is not armored */
     int inp_eof;
